@@ -16,50 +16,53 @@
 
 """Policy Engine For Glance"""
 
-# TODO(smcginnis) update this once six has support for collections.abc
-# (https://github.com/benjaminp/six/pull/241) or clean up once we drop py2.7.
-try:
-    from collections.abc import Mapping
-except ImportError:
-    from collections import Mapping
-
+from collections import abc
 import copy
 
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_policy import opts
 from oslo_policy import policy
 
 from glance.common import exception
 import glance.domain.proxy
-from glance.i18n import _
+from glance.i18n import _, _LW
+from glance import policies
 
 
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 _ENFORCER = None
 
-DEFAULT_RULES = policy.Rules.from_dict({
-    'context_is_admin': 'role:admin',
-    'default': 'role:admin',
-    'manage_image_cache': 'role:admin',
-})
+
+# TODO(gmann): Remove setting the default value of config policy_file
+# once oslo_policy change the default value to 'policy.yaml'.
+# https://github.com/openstack/oslo.policy/blob/a626ad12fe5a3abd49d70e3e5b95589d279ab578/oslo_policy/opts.py#L49
+DEFAULT_POLICY_FILE = 'policy.yaml'
+opts.set_defaults(cfg.CONF, DEFAULT_POLICY_FILE)
 
 
 class Enforcer(policy.Enforcer):
     """Responsible for loading and enforcing rules"""
 
     def __init__(self):
-        if CONF.find_file(CONF.oslo_policy.policy_file):
-            kwargs = dict(rules=None, use_conf=True)
-        else:
-            kwargs = dict(rules=DEFAULT_RULES, use_conf=False)
-        super(Enforcer, self).__init__(CONF, overwrite=False, **kwargs)
+        super(Enforcer, self).__init__(CONF, use_conf=True, overwrite=False)
+        self.register_defaults(policies.list_rules())
+        if CONF.enforce_secure_rbac and CONF.oslo_policy.enforce_new_defaults:
+            LOG.warning(_LW(
+                "Deploying glance with secure RBAC personas enabled via "
+                "`glance-api.conf [DEFAULT] enforce_secure_rbac=True` and "
+                "`glance-api.conf [oslo_policy] enforce_new_defaults=True` "
+                "is marked as EXPERIMENTAL in Wallaby. The status of this "
+                "feature will graduate to SUPPORTED as glance adopts more "
+                "personas, specifically for system-scope."
+            ))
 
     def add_rules(self, rules):
         """Add new rules to the Rules object"""
         self.set_rules(rules, overwrite=False, use_conf=self.use_conf)
 
-    def enforce(self, context, action, target):
+    def enforce(self, context, action, target, registered=True):
         """Verifies that the action is valid on the target in this context.
 
            :param context: Glance request context
@@ -68,13 +71,18 @@ class Enforcer(policy.Enforcer):
            :raises: `glance.common.exception.Forbidden`
            :returns: A non-False value if access is allowed.
         """
-        return super(Enforcer, self).enforce(action, target,
-                                             context.to_policy_values(),
-                                             do_raise=True,
-                                             exc=exception.Forbidden,
-                                             action=action)
+        if registered and action not in self.registered_rules:
+            raise policy.PolicyNotRegistered(action)
+        try:
+            return super(Enforcer, self).enforce(action, target,
+                                                 context,
+                                                 do_raise=True,
+                                                 exc=exception.Forbidden,
+                                                 action=action)
+        except policy.InvalidScope:
+            raise exception.Forbidden(action=action)
 
-    def check(self, context, action, target):
+    def check(self, context, action, target, registered=True):
         """Verifies that the action is valid on the target in this context.
 
            :param context: Glance request context
@@ -82,9 +90,11 @@ class Enforcer(policy.Enforcer):
            :param target: Dictionary representing the object of the action.
            :returns: A non-False value if access is allowed.
         """
+        if registered and action not in self.registered_rules:
+            raise policy.PolicyNotRegistered(action)
         return super(Enforcer, self).enforce(action,
                                              target,
-                                             context.to_policy_values())
+                                             context)
 
     def check_is_admin(self, context):
         """Check if the given context is associated with an admin role,
@@ -116,26 +126,74 @@ class ImageRepoProxy(glance.domain.proxy.Repo):
                                              item_proxy_kwargs=proxy_kwargs)
 
     def get(self, image_id):
+        image = super(ImageRepoProxy, self).get(image_id)
+        target = self._build_image_target(image)
         try:
-            image = super(ImageRepoProxy, self).get(image_id)
-        except exception.NotFound:
-            self.policy.enforce(self.context, 'get_image', {})
-            raise
-        else:
-            self.policy.enforce(self.context, 'get_image',
-                                dict(ImageTarget(image)))
+            self.policy.enforce(self.context, 'get_image', target)
+        except exception.Forbidden:
+            # NOTE (abhishekk): Returning 404 Not Found as the
+            # image is outside of this user's project
+            msg = _("No image found with ID %s") % image_id
+            raise exception.NotFound(msg)
         return image
 
+    def _build_image_target(self, image):
+        """Build a policy enforcement target for an image.
+
+        :param image: An instance of `glance.domain.Image`
+        :returns: a dictionary representing the image for policy enforcement
+
+        """
+        target = dict(ImageTarget(image))
+        target['project_id'] = image.owner
+
+        # We do this so that members of the image can pass the policy for
+        # getting an image shared with their project. An alternative would be
+        # to update the image owner, or project_id, to match the member ID,
+        # tricking the policy enforcer into thinking image members are actually
+        # image owners. But, that feels less clear without understanding the
+        # code that makes that assumption, especially for operators reading
+        # check strings. Using this approach forces the check_str to be more
+        # descriptive.
+        members = self.image_repo.db_api.image_member_find(
+            self.context, image_id=image.image_id)
+        # FIXME(lbragstad): Remove this if statement if/when oslo.policy
+        # supports lists of target attributes via substitution, allowing us to
+        # do something like:
+        #
+        # target['member_ids'] = set(m['member'] for m in members)
+        for member in members:
+            if member['member'] == self.context.project_id:
+                target['member_id'] = member['member']
+                break
+        return target
+
     def list(self, *args, **kwargs):
-        self.policy.enforce(self.context, 'get_images', {})
+        # FIXME(lbragstad): This is a hack to get policy to pass because we
+        # don't have a reasonable target to use for all images. We set the
+        # target project_id to the context project_id, which effectively
+        # ensures the context project_id matches itself in policy enforcement.
+        #
+        # A more accurate and cleaner way to implement this, and filtering,
+        # would be to query all images from the database, build a target for
+        # each image, and then iterate over each image and call policy
+        # enforcement. If the user passes policy enforcement, append the image
+        # to the list of filtered images. If not, the image should be removed
+        # from the list of images returned to the user.
+        target = {'project_id': self.context.project_id}
+        self.policy.enforce(self.context, 'get_images', target)
         return super(ImageRepoProxy, self).list(*args, **kwargs)
 
     def save(self, image, from_state=None):
-        self.policy.enforce(self.context, 'modify_image', dict(image.target))
+        target = dict(image.target)
+        target['project_id'] = target.get('owner', None)
+        self.policy.enforce(self.context, 'modify_image', target)
         return super(ImageRepoProxy, self).save(image, from_state=from_state)
 
     def add(self, image):
-        self.policy.enforce(self.context, 'add_image', dict(image.target))
+        target = dict(image.target)
+        target['project_id'] = target.get('owner', None)
+        self.policy.enforce(self.context, 'add_image', target)
         return super(ImageRepoProxy, self).add(image)
 
 
@@ -161,8 +219,9 @@ class ImageProxy(glance.domain.proxy.Image):
 
     @visibility.setter
     def visibility(self, value):
-        _enforce_image_visibility(self.policy, self.context, value,
-                                  self.target)
+        target = dict(self.target)
+        target['project_id'] = target.get('owner', None)
+        _enforce_image_visibility(self.policy, self.context, value, target)
         self.image.visibility = value
 
     @property
@@ -183,25 +242,31 @@ class ImageProxy(glance.domain.proxy.Image):
         self.image.locations = new_locations
 
     def delete(self):
-        self.policy.enforce(self.context, 'delete_image', dict(self.target))
+        target = dict(self.target)
+        target['project_id'] = target.get('owner', None)
+        self.policy.enforce(self.context, 'delete_image', target)
         return self.image.delete()
 
     def deactivate(self):
         LOG.debug('Attempting deactivate')
-        target = ImageTarget(self.image)
+        target = dict(ImageTarget(self.image))
+        target['project_id'] = target.get('owner', None)
         self.policy.enforce(self.context, 'deactivate', target=target)
         LOG.debug('Deactivate allowed, continue')
         self.image.deactivate()
 
     def reactivate(self):
         LOG.debug('Attempting reactivate')
-        target = ImageTarget(self.image)
+        target = dict(ImageTarget(self.image))
+        target['project_id'] = target.get('owner', None)
         self.policy.enforce(self.context, 'reactivate', target=target)
         LOG.debug('Reactivate allowed, continue')
         self.image.reactivate()
 
     def get_data(self, *args, **kwargs):
-        self.policy.enforce(self.context, 'download_image', self.target)
+        target = dict(ImageTarget(self.image))
+        target['project_id'] = target.get('owner', None)
+        self.policy.enforce(self.context, 'download_image', target)
         return self.image.get_data(*args, **kwargs)
 
     def set_data(self, *args, **kwargs):
@@ -229,8 +294,14 @@ class ImageFactoryProxy(glance.domain.proxy.ImageFactory):
                                                 proxy_kwargs=proxy_kwargs)
 
     def new_image(self, **kwargs):
+        # If we reversed the order of this method and did the policy
+        # enforcement on the way out instead of before we build the image
+        # target reference, we could use the actual image as a target instead
+        # of building a faux target with one attribute.
+        target = {}
+        target['project_id'] = kwargs.get('owner', None)
         _enforce_image_visibility(self.policy, self.context,
-                                  kwargs.get('visibility'), {})
+                                  kwargs.get('visibility'), target)
         return super(ImageFactoryProxy, self).new_image(**kwargs)
 
 
@@ -253,23 +324,46 @@ class ImageMemberRepoProxy(glance.domain.proxy.Repo):
         self.policy = policy
 
     def add(self, member):
-        self.policy.enforce(self.context, 'add_member', self.target)
+        target = dict(self.target)
+        target['project_id'] = self.context.project_id
+        self.policy.enforce(self.context, 'add_member', target)
         self.member_repo.add(member)
 
     def get(self, member_id):
-        self.policy.enforce(self.context, 'get_member', self.target)
+        # NOTE(lbragstad): We set the project_id of the target to be the
+        # project_id of the context object, which is effectively a no-op
+        # because we're checking the context.project_id matches the
+        # context.project_id. This is a bandaid to allow project-members and
+        # project-readers to view shared images owned by their project, or to
+        # view images shared with them by another project. Glance's database
+        # layer filters the images by ownership and membership, based on the
+        # context object and administrative checks. If or when that code is
+        # pulled into a higher level and if we have a list of members for an
+        # image, we can write a more accurate target.
+        target = dict(self.target)
+        # We can't set the project_id as the image project_id because that
+        # wouldn't allow image members to pass the policy check. We need the
+        # list of image members to build an accurate target.
+        target['project_id'] = self.context.project_id
+        self.policy.enforce(self.context, 'get_member', target)
         return self.member_repo.get(member_id)
 
     def save(self, member, from_state=None):
-        self.policy.enforce(self.context, 'modify_member', self.target)
+        target = dict(self.target)
+        target['project_id'] = self.context.project_id
+        self.policy.enforce(self.context, 'modify_member', target)
         self.member_repo.save(member, from_state=from_state)
 
     def list(self, *args, **kwargs):
-        self.policy.enforce(self.context, 'get_members', self.target)
+        target = dict(self.target)
+        target['project_id'] = self.context.project_id
+        self.policy.enforce(self.context, 'get_members', target)
         return self.member_repo.list(*args, **kwargs)
 
     def remove(self, member):
-        self.policy.enforce(self.context, 'delete_member', self.target)
+        target = dict(self.target)
+        target['project_id'] = self.context.project_id
+        self.policy.enforce(self.context, 'delete_member', target)
         self.member_repo.remove(member)
 
 
@@ -292,7 +386,11 @@ class ImageLocationsProxy(object):
 
     def _get_checker(action, func_name):
         def _checker(self, *args, **kwargs):
-            self.policy.enforce(self.context, action, {})
+            target = {}
+            if self.context.project_id:
+                target['project_id'] = self.context.project_id
+
+            self.policy.enforce(self.context, action, target)
             method = getattr(self.locations, func_name)
             return method(*args, **kwargs)
         return _checker
@@ -351,14 +449,17 @@ class TaskRepoProxy(glance.domain.proxy.TaskRepo):
                              task_proxy_class=TaskProxy,
                              task_proxy_kwargs=proxy_kwargs)
 
+    # TODO(lbragstad): Move this to the tasks api itself
     def get(self, task_id):
         self.policy.enforce(self.context, 'get_task', {})
         return super(TaskRepoProxy, self).get(task_id)
 
+    # TODO(lbragstad): Move this to the tasks api itself
     def add(self, task):
         self.policy.enforce(self.context, 'add_task', {})
         super(TaskRepoProxy, self).add(task)
 
+    # TODO(lbragstad): Remove this after Xena
     def save(self, task):
         self.policy.enforce(self.context, 'modify_task', {})
         super(TaskRepoProxy, self).save(task)
@@ -394,7 +495,7 @@ class TaskFactoryProxy(glance.domain.proxy.TaskFactory):
             task_proxy_kwargs=proxy_kwargs)
 
 
-class ImageTarget(Mapping):
+class ImageTarget(abc.Mapping):
     SENTINEL = object()
 
     def __init__(self, target):
@@ -405,6 +506,15 @@ class ImageTarget(Mapping):
         self.target = target
         self._target_keys = [k for k in dir(ImageProxy)
                              if not k.startswith('__')
+                             # NOTE(lbragstad): The locations attributes is an
+                             # instance of ImageLocationsProxy, which isn't
+                             # serialized into anything oslo.policy can use. If
+                             # we need to use locations in policies, we need to
+                             # modify how we represent those location objects
+                             # before we call enforcement with target
+                             # information. Omitting for not until that is a
+                             # necessity.
+                             if not k == 'locations'
                              if not callable(getattr(ImageProxy, k))]
 
     def __getitem__(self, key):
@@ -458,6 +568,10 @@ class MetadefNamespaceProxy(glance.domain.proxy.MetadefNamespace):
         self.policy = policy
         super(MetadefNamespaceProxy, self).__init__(namespace)
 
+    def delete(self):
+        self.policy.enforce(self.context, 'delete_metadef_namespace', {})
+        return super(MetadefNamespaceProxy, self).delete()
+
 
 class MetadefNamespaceRepoProxy(glance.domain.proxy.MetadefNamespaceRepo):
 
@@ -487,6 +601,14 @@ class MetadefNamespaceRepoProxy(glance.domain.proxy.MetadefNamespaceRepo):
         self.policy.enforce(self.context, 'add_metadef_namespace', {})
         return super(MetadefNamespaceRepoProxy, self).add(namespace)
 
+    def remove(self, namespace):
+        self.policy.enforce(self.context, 'delete_metadef_namespace', {})
+        return super(MetadefNamespaceRepoProxy, self).remove(namespace)
+
+    def remove_tags(self, namespace):
+        self.policy.enforce(self.context, 'delete_metadef_tags', {})
+        return super(MetadefNamespaceRepoProxy, self).remove_tags(namespace)
+
 
 class MetadefNamespaceFactoryProxy(
         glance.domain.proxy.MetadefNamespaceFactory):
@@ -511,6 +633,10 @@ class MetadefObjectProxy(glance.domain.proxy.MetadefObject):
         self.policy = policy
         super(MetadefObjectProxy, self).__init__(meta_object)
 
+    def delete(self):
+        self.policy.enforce(self.context, 'delete_metadef_object', {})
+        return super(MetadefObjectProxy, self).delete()
+
 
 class MetadefObjectRepoProxy(glance.domain.proxy.MetadefObjectRepo):
 
@@ -540,6 +666,10 @@ class MetadefObjectRepoProxy(glance.domain.proxy.MetadefObjectRepo):
         self.policy.enforce(self.context, 'add_metadef_object', {})
         return super(MetadefObjectRepoProxy, self).add(meta_object)
 
+    def remove(self, meta_object):
+        self.policy.enforce(self.context, 'delete_metadef_object', {})
+        return super(MetadefObjectRepoProxy, self).remove(meta_object)
+
 
 class MetadefObjectFactoryProxy(glance.domain.proxy.MetadefObjectFactory):
 
@@ -563,6 +693,11 @@ class MetadefResourceTypeProxy(glance.domain.proxy.MetadefResourceType):
         self.policy = policy
         super(MetadefResourceTypeProxy, self).__init__(meta_resource_type)
 
+    def delete(self):
+        self.policy.enforce(self.context,
+                            'remove_metadef_resource_type_association', {})
+        return super(MetadefResourceTypeProxy, self).delete()
+
 
 class MetadefResourceTypeRepoProxy(
         glance.domain.proxy.MetadefResourceTypeRepo):
@@ -590,6 +725,12 @@ class MetadefResourceTypeRepoProxy(
                             'add_metadef_resource_type_association', {})
         return super(MetadefResourceTypeRepoProxy, self).add(resource_type)
 
+    def remove(self, *args, **kwargs):
+        self.policy.enforce(self.context,
+                            'remove_metadef_resource_type_association', {})
+        return super(MetadefResourceTypeRepoProxy,
+                     self).remove(*args, **kwargs)
+
 
 class MetadefResourceTypeFactoryProxy(
         glance.domain.proxy.MetadefResourceTypeFactory):
@@ -614,6 +755,10 @@ class MetadefPropertyProxy(glance.domain.proxy.MetadefProperty):
         self.policy = policy
         super(MetadefPropertyProxy, self).__init__(namespace_property)
 
+    def delete(self):
+        self.policy.enforce(self.context, 'remove_metadef_property', {})
+        return super(MetadefPropertyProxy, self).delete()
+
 
 class MetadefPropertyRepoProxy(glance.domain.proxy.MetadefPropertyRepo):
 
@@ -647,6 +792,10 @@ class MetadefPropertyRepoProxy(glance.domain.proxy.MetadefPropertyRepo):
         return super(MetadefPropertyRepoProxy, self).add(
             namespace_property)
 
+    def remove(self, *args, **kwargs):
+        self.policy.enforce(self.context, 'remove_metadef_property', {})
+        return super(MetadefPropertyRepoProxy, self).remove(*args, **kwargs)
+
 
 class MetadefPropertyFactoryProxy(glance.domain.proxy.MetadefPropertyFactory):
 
@@ -669,6 +818,10 @@ class MetadefTagProxy(glance.domain.proxy.MetadefTag):
         self.policy = policy
         super(MetadefTagProxy, self).__init__(meta_tag)
 
+    def delete(self):
+        self.policy.enforce(self.context, 'delete_metadef_tag', {})
+        return super(MetadefTagProxy, self).delete()
+
 
 class MetadefTagRepoProxy(glance.domain.proxy.MetadefTagRepo):
 
@@ -702,6 +855,10 @@ class MetadefTagRepoProxy(glance.domain.proxy.MetadefTagRepo):
         self.policy.enforce(self.context, 'add_metadef_tags', {})
         return super(MetadefTagRepoProxy, self).add_tags(meta_tags)
 
+    def remove(self, meta_tag):
+        self.policy.enforce(self.context, 'delete_metadef_tag', {})
+        return super(MetadefTagRepoProxy, self).remove(meta_tag)
+
 
 class MetadefTagFactoryProxy(glance.domain.proxy.MetadefTagFactory):
 
