@@ -12,12 +12,13 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+from unittest import mock
 import uuid
 
 from cursive import exception as cursive_exception
 import glance_store
 from glance_store._drivers import filesystem
-import mock
+from oslo_config import cfg
 import six
 from six.moves import http_client as http
 import webob
@@ -30,6 +31,9 @@ from glance.tests.unit import base
 import glance.tests.unit.utils as unit_test_utils
 import glance.tests.utils as test_utils
 
+CONF = cfg.CONF
+CONF.import_opt('public_endpoint', 'glance.api.versions')
+
 
 class Raise(object):
 
@@ -44,7 +48,7 @@ class FakeImage(object):
 
     def __init__(self, image_id=None, data=None, checksum=None, size=0,
                  virtual_size=0, locations=None, container_format='bear',
-                 disk_format='rawr', status=None):
+                 disk_format='rawr', status=None, owner=None):
         self.image_id = image_id
         self.data = data
         self.checksum = checksum
@@ -53,7 +57,9 @@ class FakeImage(object):
         self.locations = locations
         self.container_format = container_format
         self.disk_format = disk_format
+        self.owner = owner
         self._status = status
+        self.extra_properties = {}
 
     @property
     def status(self):
@@ -71,7 +77,7 @@ class FakeImage(object):
             return self.data[offset:offset + chunk_size]
         return self.data[offset:]
 
-    def set_data(self, data, size=None, backend=None):
+    def set_data(self, data, size=None, backend=None, set_active=True):
         self.data = ''.join(data)
         self.size = size
         self.status = 'modified-by-fake'
@@ -175,7 +181,7 @@ class TestImagesController(base.StoreClearingUnitTest):
 
     def test_upload(self):
         request = unit_test_utils.get_fake_request()
-        image = FakeImage('abcd')
+        image = FakeImage('abcd', owner='tenant1')
         self.image_repo.result = image
         self.controller.upload(request, unit_test_utils.UUID2, 'YYYY', 4)
         self.assertEqual('YYYY', image.data)
@@ -209,12 +215,15 @@ class TestImagesController(base.StoreClearingUnitTest):
     @mock.patch.object(glance.api.policy.Enforcer, 'enforce')
     def test_upload_image_forbidden(self, mock_enforce):
         request = unit_test_utils.get_fake_request()
+        image = FakeImage('abcd', owner='tenant1')
+        self.image_repo.result = image
         mock_enforce.side_effect = exception.Forbidden
         self.assertRaises(webob.exc.HTTPForbidden, self.controller.upload,
                           request, unit_test_utils.UUID2, 'YYYY', 4)
+        expected_target = {'project_id': 'tenant1'}
         mock_enforce.assert_called_once_with(request.context,
                                              "upload_image",
-                                             {})
+                                             expected_target)
 
     def test_upload_invalid(self):
         request = unit_test_utils.get_fake_request()
@@ -372,59 +381,6 @@ class TestImagesController(base.StoreClearingUnitTest):
                           self.controller.upload,
                           request, unit_test_utils.UUID2, 'YY', 2)
 
-    @mock.patch("glance.common.trust_auth.TokenRefresher")
-    def test_upload_with_trusts(self, mock_refresher):
-        """Test that uploading with registry correctly uses trusts"""
-        # initialize trust environment
-        self.config(data_api='glance.db.registry.api')
-        refresher = mock.MagicMock()
-        mock_refresher.return_value = refresher
-        refresher.refresh_token.return_value = "fake_token"
-        # request an image upload
-        request = unit_test_utils.get_fake_request()
-        request.environ['keystone.token_auth'] = mock.MagicMock()
-        request.environ['keystone.token_info'] = {
-            'token': {
-                'roles': [{'name': 'FakeRole', 'id': 'FakeID'}]
-            }
-        }
-        image = FakeImage('abcd')
-        self.image_repo.result = image
-        mock_fake_save = mock.Mock()
-        mock_fake_save.side_effect = [None, exception.NotAuthenticated, None]
-        temp_save = FakeImageRepo.save
-        # mocking save to raise NotAuthenticated on the second call
-        FakeImageRepo.save = mock_fake_save
-        self.controller.upload(request, unit_test_utils.UUID2, 'YYYY', 4)
-        # check image data
-        self.assertEqual('YYYY', image.data)
-        self.assertEqual(4, image.size)
-        FakeImageRepo.save = temp_save
-        # check that token has been correctly acquired and deleted
-        mock_refresher.assert_called_once_with(
-            request.environ['keystone.token_auth'],
-            request.context.project_id, ['FakeRole'])
-        refresher.refresh_token.assert_called_once_with()
-        refresher.release_resources.assert_called_once_with()
-        self.assertEqual("fake_token", request.context.auth_token)
-
-    @mock.patch("glance.common.trust_auth.TokenRefresher")
-    def test_upload_with_trusts_fails(self, mock_refresher):
-        """Test upload with registry if trust was not successfully created"""
-        # initialize trust environment
-        self.config(data_api='glance.db.registry.api')
-        mock_refresher().side_effect = Exception()
-        # request an image upload
-        request = unit_test_utils.get_fake_request()
-        image = FakeImage('abcd')
-        self.image_repo.result = image
-        self.controller.upload(request, unit_test_utils.UUID2, 'YYYY', 4)
-        # check image data
-        self.assertEqual('YYYY', image.data)
-        self.assertEqual(4, image.size)
-        # check that the token has not been updated
-        self.assertEqual(0, mock_refresher().refresh_token.call_count)
-
     def _test_upload_download_prepare_notification(self):
         request = unit_test_utils.get_fake_request()
         self.controller.upload(request, unit_test_utils.UUID2, 'YYYY', 4)
@@ -606,6 +562,49 @@ class TestImagesController(base.StoreClearingUnitTest):
         self.assertRaises(webob.exc.HTTPConflict, self.controller.stage,
                           request, image_id, 'YYYY', 4)
 
+    def _test_image_stage_records_host(self, expected_url):
+        image_id = str(uuid.uuid4())
+        request = unit_test_utils.get_fake_request()
+        image = FakeImage(image_id=image_id)
+        self.image_repo.result = image
+        with mock.patch.object(filesystem.Store, 'add'):
+            self.controller.stage(request, image_id, 'YYYY', 4)
+        if expected_url is None:
+            self.assertNotIn('os_glance_stage_host', image.extra_properties)
+        else:
+            self.assertEqual(expected_url,
+                             image.extra_properties['os_glance_stage_host'])
+
+    def test_image_stage_records_host_unset(self):
+        # Make sure we do not set a null staging host, if we are not configured
+        # to support worker-to-worker communication.
+        self._test_image_stage_records_host(None)
+
+    def test_image_stage_records_host_public_endpoint(self):
+        # Make sure we fall back to public_endpoint
+        self.config(public_endpoint='http://lb.example.com')
+        self._test_image_stage_records_host('http://lb.example.com')
+
+    def test_image_stage_records_host_self_url(self):
+        # Make sure worker_self_reference_url takes precedence
+        self.config(worker_self_reference_url='http://worker1.example.com')
+        self._test_image_stage_records_host('http://worker1.example.com')
+
+    def test_image_stage_fail_does_not_set_host(self):
+        # Make sure that if the store.add() fails, we do not claim to have the
+        # image staged.
+        self.config(public_endpoint='http://worker1.example.com')
+        image_id = str(uuid.uuid4())
+        request = unit_test_utils.get_fake_request()
+        image = FakeImage(image_id=image_id)
+        self.image_repo.result = image
+        exc_cls = glance_store.exceptions.StorageFull
+        with mock.patch.object(filesystem.Store, 'add', side_effect=exc_cls):
+            self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
+                              self.controller.stage,
+                              request, image_id, 'YYYY', 4)
+        self.assertNotIn('os_glance_stage_host', image.extra_properties)
+
 
 class TestImageDataDeserializer(test_utils.BaseTestCase):
 
